//=======================================================================
//			Copyright (C) Shambler Team 2004
//		         util_.cpp - utility entities: util_fade, 
//			       util_shake, util_fov etc			    
//=======================================================================

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "gamerules.h"

//=======================================================================
// func_mirror - breakable mirror brush 
//=======================================================================
extern int gmsgAddMirror;

class CFuncMirror : public CBaseEntity
{
public:
	void Spawn( void );
	void PostActivate( void );
	void KeyValue( KeyValueData *pkvd );
	void Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );
	virtual int ObjectCaps( void ) { return CBaseEntity :: ObjectCaps() & ~FCAP_ACROSS_TRANSITION; }
};
LINK_ENTITY_TO_CLASS( func_mirror, CFuncMirror );

void CFuncMirror::KeyValue( KeyValueData *pkvd )
{
	if (FStrEq(pkvd->szKeyName, "alpha"))
	{
		pev->scale = atof( pkvd->szValue );
		pkvd->fHandled = TRUE;
	}
	else	
		CBaseEntity::KeyValue( pkvd );
}

void CFuncMirror :: Spawn( void )
{
	pev->flags = FL_WORLDBRUSH;

	if ( g_pGameRules->IsMultiplayer() )
	{
		ALERT(at_console, "\n======/Xash SmartFiled System/======\n\n");
		ALERT(at_console, "Sorry, %s don't working in multiplayer\nPlease, use static world mirrors\n", STRING(pev->classname));
	}

	SET_MODEL( ENT(pev), STRING(pev->model) );
	if(pev->spawnflags & 2)//make illusionary wall 
	{
		pev->solid = SOLID_NOT;
		pev->movetype = MOVETYPE_NONE;
	}
	else
	{
		pev->solid = SOLID_BSP;
        pev->movetype = MOVETYPE_PUSH;
	}
	pev->body = TRUE;
}

void CFuncMirror :: PostActivate( void )
{
	MESSAGE_BEGIN( MSG_ALL, gmsgAddMirror );
		WRITE_BYTE( entindex() );
	MESSAGE_END();
}

void CFuncMirror :: Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{
	if (useType == USE_TOGGLE)
	{
		if(pev->body) useType = USE_OFF;
		else useType = USE_ON;
	}
	if (useType == USE_ON) pev->body = TRUE;
	else if (useType == USE_OFF)pev->body = FALSE;
}

//=======================================================================
// func_monitor - A monitor that renders the view from a camera entity. 
//=======================================================================
extern int gmsgAddScreen;

class CFuncMonitor : public CBaseToggle
{
public:
	void Spawn( void );
	void PostActivate ( void );
	void KeyValue( KeyValueData *pkvd );
	void Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );
	virtual int ObjectCaps( void );
	CBaseEntity *pCamera;
	CBaseEntity *m_pPlayer;
};
LINK_ENTITY_TO_CLASS( func_monitor, CFuncMonitor );

void CFuncMonitor::KeyValue( KeyValueData *pkvd )
{
	if (FStrEq(pkvd->szKeyName, "camera"))
	{
		pev->netname = ALLOC_STRING( pkvd->szValue );
		pkvd->fHandled = TRUE;
	}
	if (FStrEq(pkvd->szKeyName, "type"))
	{
		pev->impulse = atoi( pkvd->szValue );
		pkvd->fHandled = TRUE;
	}
	if (FStrEq(pkvd->szKeyName, "mode"))
	{
		pev->skin = atoi( pkvd->szValue );
		pkvd->fHandled = TRUE;
	}
	else	
		CBaseEntity::KeyValue( pkvd );
}

void CFuncMonitor :: Spawn()
{
	if(pev->spawnflags & 2)//make illusionary wall 
	{
		pev->solid = SOLID_NOT;
		pev->movetype = MOVETYPE_NONE;
	}
	else
	{
		pev->solid = SOLID_BSP;
        pev->movetype = MOVETYPE_PUSH;
	}
	SET_MODEL( ENT(pev), STRING(pev->model) );

	//enable monitor
	if(pev->spawnflags & 1)
		pev->body = 1;
}

int CFuncMonitor :: ObjectCaps( void ) 
{
	if (pev->impulse)
		return (CBaseEntity::ObjectCaps() & ~FCAP_ACROSS_TRANSITION) | FCAP_IMPULSE_USE;
	else	
		return (CBaseEntity::ObjectCaps() & ~FCAP_ACROSS_TRANSITION);
}

void CFuncMonitor::PostActivate( void )
{
	//send monitor index
	if(!pCamera)
	{
		MESSAGE_BEGIN( MSG_ALL, gmsgAddScreen );
			WRITE_BYTE( entindex() );
		MESSAGE_END();
	}
	pCamera = UTIL_FindEntityByTargetname(NULL, STRING(pev->netname));
    if(pCamera)pev->scale = pCamera->entindex();
	else if(pev->body) pev->frame = 1;//noise if not found camera
}

void CFuncMonitor :: Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{
	m_pPlayer = pActivator;
	
	if (!UTIL_IsMasterTriggered(m_sMaster, m_pPlayer))
		return;

	if (useType == USE_TOGGLE)
	{
		if(pev->body) useType = USE_OFF;
		else useType = USE_ON;
	}
	if (useType == USE_ON)pev->body = 1;
	else if (useType == USE_OFF)
	{
		pev->body = 0;
		pev->frame = 0;
	}
}

